package VolansCell;

import javax.microedition.io.*;
import java.util.Date;


/*
 * VolansCell - Version 6
 * (c) 2010 Chris VanHorn
 */
 public class MainThread extends Thread{

    // Current conditions
    private int count;                              // Itteration Count
    private int running;                            // Should we keep this thread running?
    private long lasttimestamp;                     // Timestamp of the last iterreration
    private int sleeptime;                          // How long to sleep on this itteration
    private Runtime rt;                             // Current runtime information
    private int lastsignal;                         // Last signal level
    private int lastbattery;                        // Last battery level
    private int lastaccuracy;                       // Last GPS accuracy
    private DataCollection datacollectionthread;    // DataCollection thread

    // Configuration variables
    private String url;                             // URL to send updates to
    private int gpspollms;                          // GPS poll interval
    private String cellname;                        // Cell name
    private String cellpassword;                    // Cell password

    // Threads
    private VolansMain volansmainthread;            // VolansMain thread

    // *************************************************************************
    public MainThread(VolansMain temp) {

        // Save our parrent thread
        volansmainthread = temp;

        // Set some default values
        count = 0;
        running = 1;
        lastsignal = -1;
        lastbattery = -1;
        lastaccuracy = -1;

        // Configuration
        url = volansmainthread.getAppProperty("VolansURL");
        gpspollms = Integer.parseInt(volansmainthread.getAppProperty("VolansGPSPollMS"));
        cellname = volansmainthread.getAppProperty("VolansCellName");
        cellpassword = volansmainthread.getAppProperty("VolansCellPassword");

        // Open the runtime
        rt = Runtime.getRuntime();
    }
    // *************************************************************************


    // *************************************************************************
    public void run() {

        // Main loop
        while (running == 1) {

            // Increment out counter + timestamp
            count++;
            lasttimestamp = System.currentTimeMillis();

            // Create an alarm to run 3 minutes from now if application dies
            try {
                long restarttime = new Date().getTime() + (1000 * 180);
                PushRegistry.registerAlarm("VolansCell.VolansMain", restarttime);
            } catch (Exception e) {
            }

            // Resume VolansMain if it's paused and the flip is open
            try {
                if (volansmainthread.ispaused == 1) {
                    synchronized(volansmainthread.externaldisplay) {
                        if (volansmainthread.externaldisplay == null)
                            volansmainthread.resumeRequest();
                        else if (!volansmainthread.externaldisplay.getFlipState())
                            volansmainthread.resumeRequest();
                    }
                }
            } catch (Exception e) {
            }

            // Update the display
            try {
                synchronized(volansmainthread.theform) {
                    volansmainthread.theform.deleteAll();
                    volansmainthread.theform.append(cellname + "\n");
                    volansmainthread.theform.append("Count: " + Integer.toString(count) + "\n");
                    volansmainthread.theform.append("Signl: " + Integer.toString(lastsignal) + "/4\n");
                    volansmainthread.theform.append("Batt : " + Integer.toString(lastbattery) + "/3\n");
                    volansmainthread.theform.append("Mem  : " + Integer.toString(getFreeMemKB()) + "kB\n");
                    if (lastaccuracy == -1)
                        volansmainthread.theform.append("GPS  : None");
                    else
                        volansmainthread.theform.append("GPS  : " + Integer.toString(lastaccuracy) + "m");
                }
            } catch (Exception e) {
            }

            // Stop the current DataCollection thread if it's stuck
            try {
                if (datacollectionthread != null)
                    if (datacollectionthread.getState().equals("HTTPResp"))
                        datacollectionthread.killme();
            } catch (Exception e) {
            }

            // Kick off DataCollection thread
            try {
                datacollectionthread = new DataCollection(this, cellname, cellpassword, url, count);
                datacollectionthread.start();
            } catch (Exception e) {
                datacollectionthread = null;
            }


            // Ask for garbage collection every 10 itterations
            if (count % 10 == 0)
                System.gc();


            // Pause for poll amount
            try {
                sleeptime = (int)(gpspollms - (int)(System.currentTimeMillis() - lasttimestamp));
                if (sleeptime < 10) sleeptime = 10;
                this.sleep(sleeptime);
            } catch (Exception e) {
            }
        }


        // Create an alarm to run 3 minutes from now if application dies
        try {
            long restarttime = new Date().getTime() + (1000 * 180);
            PushRegistry.registerAlarm("VolansCell.VolansMain", restarttime);
        } catch (Exception e) {
        }

        // Kill off everything
        volansmainthread.notifyDestroyed();

    }
    // *************************************************************************


    // *************************************************************************
    public void killme() {
        synchronized (this) {
            running = 0;
        }
    }
    // *************************************************************************


    // *************************************************************************
    public int getRunning() {
        synchronized (this) {
            return running;
        }
    }
    // *************************************************************************


    // *************************************************************************
    public void setLastSignal(int temp) {
        synchronized (this) {
            lastsignal = temp;
        }
    }
    // *************************************************************************


    // *************************************************************************
    public void setLastBattery(int temp) {
        synchronized (this) {
            lastbattery = temp;
        }
    }
    // *************************************************************************


    // *************************************************************************
    public void setLastAccuracy(int temp) {
        synchronized (this) {
            lastaccuracy = temp;
        }
    }
    // *************************************************************************


    // *************************************************************************
    public int getFreeMemKB() {
        synchronized (this) {
            return (int)(rt.freeMemory() / 1024);
        }
    }
    // *************************************************************************

}
