package VolansCell;

import javax.microedition.midlet.*;
import javax.microedition.io.*;
import java.util.Date;
import javax.microedition.lcdui.*;
import com.motorola.iden.lcdui.ExternalDisplay;

/*
 * VolansCell - Version 6
 * (c) 2010 Chris VanHorn
 * 
 * This application will run on many Motorola IDEN Java based cell phones.
 * It will continually send the GPS coordinates of the device to the website
 * of your choice.  
 * 
 * The VolansCell.jad file has several properties that should be set.  All of
 * the properties should only contain basic ascii characters and no spaces. 
 * The URL requires a ? at the end.
 * 
 * VolansCellName       This is the name of the phone to be transmitted to
 *                      the web server.
 * VolansCellPassword   This is the password to be transmitted to the web
 *                      server.
 * VolansURL            This is the URL to transmit the phones data to.  It
 *                      should end with a ?.  For example:
 *                      http://happicow.com/phone.php?
 * VolansGPSPollMS      This is how many miliseconds it should wait between
 *                      collecting and sending data.  30000 for example would
 *                      cuase the phone to send data every 30 seconds.  If 
 *                      you set it to 0, it will send data as often as
 *                      possible.  But it is unlikely to send data faster than
 *                      about once every 10 seconds.
 *
 * The data is sent via a GET statement to keep things simple.  It does not
 * expect the return of any data from the web server.  Any data in the reply
 * is disregarded.  The properties that are sent via the GET command are
 * described below.
 * 
 * cellname         VolansCell.jad VolansCellName property value
 * cellpassword     VolansCell.jad VolansCellPassword property value
 * count            The number of data points sent since last restart of the
 *                  application.
 * timestamp        UNIX style, integer timestamp of the time datacollection
 *                  began.
 * signal           Strength of the phone's signal.  Range is 0 .. 4
 * battery          Battery level.  Range is 0 .. 3
 * locationtype     Indicates what the location in lat and lon values is.
 *                  It can have the following values and their meanings:
 *                      CELL - location is the cell tower we are talking to
 *                      GPS - location is the phone's location from GPS
 *                      UNKNOWN - we don't know where we got the location from
 *                  The GPS system on these phones does not always respond
 *                  in time.  And some phones will turn it off if the battery
 *                  level drops below a threshold.  In which case it will
 *                  display CELL or UNKNOWN
 * latitude         Location latitude in hh.ddddd format
 *                  (some rounding may be necessary)
 * longitude        Location longitude in hhh.ddddd format
 *                  (some rounding may be necessary)
 * positionaccuracy Accuracy of the lat and lon values in meters
 * altitude         Altitude of the phone in meters
 * altitudeaccuracy Accuracy of the altitude value in meters
 * heading          Heading in degrees
 * speed            Speed in m/s
 * debug            This contains some small debugging information in the
 *                  format:
 *                  count,messages
 */


public class VolansMain extends MIDlet {

    // Display stuff
    public Display display;                                 // The display for this MIDlet
    public Form theform;                                    // Form that will contain our output
    public ExternalDisplay externaldisplay;                 // The external display on the flip

    // Local stuff
    public int ispaused;                                    // Paused status

    // MainThread
    private MainThread thethread;                           // Main thread

    // *************************************************************************
    public VolansMain() {

        // First thing - get the display
        try {
            display = Display.getDisplay(this);
        } catch (Exception e) {
        }


        // Get the external display
        try {
            externaldisplay = ExternalDisplay.getDisplay(this);
        } catch (Exception e) {
            externaldisplay = null;
        }


        // Default to not-paused
        ispaused = 0;


        // Create an alarm to run 5 minutes from now if application dies
        try {
            long restarttime = new Date().getTime() + (1000 * 300);
            PushRegistry.registerAlarm("VolansCell.VolansMain", restarttime);
        } catch (Exception e) {
        }
    }
    // *************************************************************************



    // *************************************************************************
    public void startApp() {

        // Set status to not paused
        ispaused = 0;


        // Create an alarm to run 3 minutes from now if application dies
        try {
            long restarttime = new Date().getTime() + (1000 * 180);
            PushRegistry.registerAlarm("VolansCell.VolansMain", restarttime);
        } catch (Exception e) {
            theform.deleteAll();
            theform.append("startApp: " + e.toString());
        }


        // Create a new form if necessary
        try {
            if (theform == null) {
                theform = new Form("VolansCell v6");
                theform.append("Loading...");
            }
        } catch (Exception e) {
        }

        // Start the screen
        try {
            display.setCurrent(theform);
            synchronized (theform) {
                theform.deleteAll();
                theform.append("Loading...");
            }
        } catch (Exception e) {
        }


        // Create our MainThread
        try {
            if (thethread == null || thethread.getRunning() == 0) {
               thethread = new MainThread(this);
                thethread.start();
            }
        } catch (Exception e) {
            theform.deleteAll();
            theform.append("StartApp: " + e.toString());
            thethread = new MainThread(this);
            thethread.start();
        }

    }
    // *************************************************************************



    // *************************************************************************
    public void pauseApp() {
        ispaused = 1;

        // Create an alarm to run 3 minutes from now if application dies
        try {
            long restarttime = new Date().getTime() + (1000 * 180);
            PushRegistry.registerAlarm("VolansCell.VolansMain", restarttime);
        } catch (Exception e) {
        }

    }
    // *************************************************************************



    // *************************************************************************
    public void destroyApp(boolean unconditional) {

        // Create an alarm to run 3 minutes from now if application dies
        try {
            long restarttime = new Date().getTime() + (1000 * 180);
            PushRegistry.registerAlarm("VolansCell.VolansMain", restarttime);
        } catch (Exception e) {
        }

    }
    // *************************************************************************

}